
//VERSION 1

//-----------------------------------------------------------------------------
// GPIO COMMANDS --------------------------------------------------------------
//-----------------------------------------------------------------------------


//description:   Turn module on
//input:         none
//return:        none
void gpio_turnOn() {
  debug_printP( PSTR("GPIO: TurnOn"),       LEVEL_INFO, 1, 1 );
  delay(100);
  digitalWrite(pin_po,  LOW);
  delay(100);
  digitalWrite(pin_po,  HIGH);
  delay(30000);
}

//description:   Turn module off
//input:         none
//return:        none
void gpio_turnOff() {
  debug_printP( PSTR("GPIO: TurnOff"),      LEVEL_INFO, 1, 1 );
  delay(100);
  digitalWrite(pin_po,  LOW);
  delay(5100);
  digitalWrite(pin_po,  HIGH);
}

//description:   Toggle module power
//input:         none
//return:        none
void gpio_toggle() {
  debug_printP( PSTR("GPIO: Toggle"),       LEVEL_INFO, 1, 1 );
  delay(100);
  digitalWrite(pin_prn,  LOW);
  delay(200);
  digitalWrite(pin_prn,  HIGH);
  delay(30000);
}

byte gpio_ledRead(byte ledNum) {
  //This yields a resolution between readings of: 5 volts / 1024 units or, .0049 volts (4.9 mV) per unit
  int led = analogRead(ledNum);       // read the input pin
  debug_printP( PSTR("GPIO: ledRead "),     LEVEL_DEBUG, 1, 0 );
  debug_print( String(ledNum),              LEVEL_DEBUG, 0, 0 );
  debug_printP( PSTR(" - "),                LEVEL_DEBUG, 0, 0 );
  debug_print( String(led),                 LEVEL_DEBUG, 0, 1 );
  if (led > LED_THRESHOLD) return 1;
  return 0;
}

//description:   Check if module is connected to a network or not
//input:         none
//return:        0=2G or 3G, 1=unregistered
byte gpio_ledConnected() {
  debug_printP( PSTR("GPIO: LED Connected"), LEVEL_INFO, 1, 1 );
  byte gpio_ledB = gpio_ledRead(pin_ledB);                      //3G
  //byte gpio_ledR = gpio_ledRead(pin_ledR);                      //Unregistered
  byte gpio_ledG = gpio_ledRead(pin_ledG);                      //2G
  if ((gpio_ledG == 0) or (gpio_ledB == 0)) {
    debug_printP( PSTR(" Connected to a cellular network"), LEVEL_INFO, 1, 1 );
    return 0;
  }
  else {
    debug_printP( PSTR(" SIM Unregistered"), LEVEL_INFO, 1, 1 );
    return 1;
  }
}
        
//description:   Check if module power is on (one of the LEDs should be lit with power)
//input:         none
//return:        0=power,1=off
byte gpio_ledLit() {
  debug_printP( PSTR("GPIO: LED Lit"),      LEVEL_INFO, 1, 1 );
  byte gpio_ledB = gpio_ledRead(pin_ledB);                      //3G
  byte gpio_ledR = gpio_ledRead(pin_ledR);                      //Unregistered
  byte gpio_ledG = gpio_ledRead(pin_ledG);                      //2G
  
  if ((gpio_ledG == 0) or (gpio_ledB == 0) or (gpio_ledR == 0)) {
    debug_printP( PSTR(" On"),              LEVEL_INFO, 1, 1 );
    return 0;
  }
  else {
    debug_printP( PSTR(" Off"),             LEVEL_INFO, 1, 1 );
    return 1;
  }
}

//description:   Module Wakeup AP
//input:         none
//return:        1=Wake AP, 0=AP Awake
byte gpio_statusMWA() {
  debug_printP( PSTR("GPIO: statusMWA"),    LEVEL_INFO, 1, 1 );
  int gpio_mwa = digitalRead(pin_mwa);
  
  if (gpio_mwa == 0) { 
    debug_printP( PSTR(" PIN:MWA - Wake AP"), LEVEL_INFO, 1, 1 );
    return 1;
  }
  else {
    debug_printP( PSTR(" PIN:MWA - AP Awake"), LEVEL_INFO, 1, 1 );
    return 0;
  }
}

//description:   Module Poweron
//input:         none
//return:        1=off, 0=on
byte gpio_statusMP() {
  debug_printP( PSTR("GPIO: statusMP"),     LEVEL_INFO, 1, 1 );
  int gpio_mp = digitalRead(pin_mp);
  
  if (gpio_mp == 0) {
    debug_printP( PSTR(" PIN:MP - Off"),    LEVEL_INFO, 1, 1 );
    return 1;
  }
  else {
    debug_printP( PSTR(" PIN:MP - On"),     LEVEL_INFO, 1, 1 );
    return 0;
  }
}
        
//description:   Module Ready
//input:         none
//return:        0=awake, 1=asleep 
byte gpio_statusMR() {
  debug_printP( PSTR("GPIO: statusMR"),     LEVEL_INFO, 1, 1 );
  int gpio_mr = digitalRead(pin_mr);
  
  if (gpio_mr == 0) {
    debug_printP( PSTR(" PIN:MR - Awake"),  LEVEL_INFO, 1, 1 );
    return 0;
  }
  else {
    debug_printP( PSTR(" PIN:MR - Asleep"), LEVEL_INFO, 1, 1 );
    return 1;
  }
}
        
//-----------------------------------------------------------------------------
// DEBUG COMMANDS -------------------------------------------------------------
//-----------------------------------------------------------------------------

//description:   Print state of network LEDs
//input:         none
//return:        none
void debug_led() {
  debug_printP( PSTR("LED"),                LEVEL_DEBUG, 1, 1 );
  byte ledR = gpio_ledRead(pin_ledR);
  byte ledG = gpio_ledRead(pin_ledG);
  byte ledB = gpio_ledRead(pin_ledB);
  
  debug_printP( PSTR(" ledR:"),             LEVEL_DEBUG, 1, 0 );
  debug_print( String(ledR),                LEVEL_DEBUG, 0, 0 );
  debug_printP( PSTR(" Red   - Not Connected (0 on, 1 off)"), LEVEL_DEBUG, 0, 1);
  
  debug_printP( PSTR(" ledG:"),             LEVEL_DEBUG, 1, 0 );
  debug_print( String(ledG),                LEVEL_DEBUG, 0, 0 );
  debug_printP( PSTR(" Green - 2G (0 on, 1 off)"), LEVEL_DEBUG, 0, 1);
  
  debug_printP( PSTR(" ledB:"),             LEVEL_DEBUG, 1, 0 );
  debug_print( String(ledB),                LEVEL_DEBUG, 0, 0 );
  debug_printP( PSTR(" Blue  - 3G (0 on, 1 off)"), LEVEL_DEBUG, 0, 1);
}

//description:   Print state of status LEDs
//input:         none
//return:        none
void debug_inputs() {
  debug_printP( PSTR("Inputs"),             LEVEL_DEBUG, 1, 1 );
  
  debug_printP( PSTR(" Module Wakeup AP: "), LEVEL_DEBUG, 1, 0 );
  debug_print( String(digitalRead(pin_mwa)), LEVEL_DEBUG, 0, 0 );
  debug_printP( PSTR("(0 wake AP, 1 AP awake)"), LEVEL_DEBUG, 0, 1 );
    
  debug_printP( PSTR(" Module Poweron: "),  LEVEL_DEBUG, 1, 0 );
  debug_print( String(digitalRead(pin_mp)), LEVEL_DEBUG, 0, 0 );
  debug_printP( PSTR("(0 off, 1 power)"),   LEVEL_DEBUG, 0, 1 );
    
  debug_printP( PSTR(" Module Ready: "),    LEVEL_DEBUG, 1, 0 );
  debug_print( String(digitalRead(pin_mr)), LEVEL_DEBUG, 0, 0 );
  debug_printP( PSTR("(0 awake, 1 asleep)"),LEVEL_DEBUG, 0, 1 );
}
    
//description:   debug print function - debug level for multiline statements
//input:         none
//return:        none
void debug_printLevel(byte level) {
  if (LEVEL_CRITICAL == level)     debug_P(PSTR("CRITICAL:  "));
  else if (LEVEL_ERROR == level)   debug_P(PSTR("ERROR:     "));
  else if (LEVEL_WARNING == level) debug_P(PSTR("WARNING:   "));
  else if (LEVEL_INFO == level)    debug_P(PSTR("INFO:      "));
  else if (LEVEL_DEBUG == level)   debug_P(PSTR("DEBUG:     "));
  else                             debug_P(PSTR("UNDEFINED: "));
}

//description:   debug print function - single string for multiline statements
//input:         none
//return:        none
void debug_print(String string, byte level, byte in, byte out) {
  if (level <= glob_debug) {
    if (in == 1) debug_printLevel(level);
    mySerial.print(string);
    if (out == 1) mySerial.println("");
  }
}
//description:   debug print function - character if >0x20 && <0x88, hex otherwise
//input:         none
//return:        none
void debug_printSafe(char str, byte level) {
  if (level <= glob_debug) {
    if ((str < 0x20) || (str > 0x7e)) {
      mySerial.print (" 0x");
      mySerial.print (str, HEX);
      mySerial.print (" ");
    }
    else
      mySerial.print(str);
  }
}


//-----------------------------------------------------------------------------
// PROGMEM COMMANDS - created in order to keep RAM usage low ------------------
//-----------------------------------------------------------------------------

//description:   debug print function - single string for multiline statements (PROGMEM)
//input:         none
//return:        none
void debug_printP(const char *str, byte level, byte in, byte out) {
  if (level <= glob_debug) {
    if (in == 1) debug_printLevel(level);
    debug_P(str);
    if (out == 1) mySerial.println("");
  }
}
//description:   print (PROGMEM)
//input:         none
//return:        none
void debug_P(const char *str) {
  while (pgm_read_byte(str) != 0x00) {
    char c = (char)pgm_read_byte(str++);
      mySerial.print(c);
  }
}


//-----------------------------------------------------------------------------
// INFO COMMANDS --------------------------------------------------------------
//-----------------------------------------------------------------------------

//description:   checks the registration status of the SIM card.  
//               Make or break function - must pass this or app should fail
//input:         none
//return:        0=no issue (registered), 1=not registered, needs to alert user
byte info_status() {                  //+CREG: 0,5 ... OK
  debug_printP( PSTR("Status"),             LEVEL_DEBUG, 1, 1 );
  byte tries = 3;
  while (tries > 0) {
    tries -= 1;
    byte tries1 = 3;
    while (tries1 > 0) {
      tries1 -= 1;
      int cmd_ret = cmd_full("at+creg?\r", 3, 1000, "CREG:");            //get "at+creg?" response "CREG:"
      if (cmd_ret != -1) {
        debug_printP(PSTR("CREG Index: ->"), LEVEL_DEBUG, 1, 0);
        String re_ret = buffer.substring(cmd_ret + 8,cmd_ret + 9);
        debug_print(re_ret,                 LEVEL_DEBUG, 0, 0);
        debug_printP(PSTR("<-"),            LEVEL_DEBUG, 0, 1);
        
        if ((re_ret == "0") or (re_ret == "3")) 
          debug_printP( PSTR(" SIM card not registering with current network"), LEVEL_ERROR, 1, 1 );
        else if ((re_ret == "1") or (re_ret == "5")) {
          debug_printP( PSTR(" SIM registered"), LEVEL_INFO, 1, 1 );
          return 0;
        }
        else if (re_ret == "2")
          debug_printP( PSTR(" SIM still trying to register"), LEVEL_INFO, 1, 1 );
        else if (re_ret == "4")
          debug_printP( PSTR(" SIM registration unknown"), LEVEL_DEBUG, 1, 1 );
        else
          debug_printP( PSTR(" Unknown response"), LEVEL_ERROR, 1, 1 );
      }
      else
        debug_printP( PSTR(" Unable to retrieve SIM card registration"), LEVEL_ERROR, 1, 1 );
    }
    gpio_toggle();
  }
  return 1;
}

//-----------------------------------------------------------------------------
// PROCESSING COMMANDS --------------------------------------------------------
//-----------------------------------------------------------------------------

//description:   Setup GPIO for RasPi shield
//input:         none
//return:        none
void cmd_setup() {
  debug_printP( PSTR("CMD Status"),         LEVEL_DEBUG, 1, 1 );
  Serial.begin(115200);
  
  pinMode(pin_prn, OUTPUT);
  pinMode(pin_mwa, INPUT);
  pinMode(pin_mp,  INPUT);
  pinMode(pin_mr,  INPUT);
  pinMode(pin_ar,  OUTPUT);
  pinMode(pin_awm, OUTPUT);
  pinMode(pin_po,  OUTPUT);
  
  digitalWrite(pin_prn, HIGH);            //no reset (toggle to reset)
  digitalWrite(pin_ar,  LOW);             //AP is awake
  digitalWrite(pin_awm, LOW);             //AP wakes up module
  digitalWrite(pin_po,  HIGH);            //high is awake (toggle later to turn on/off)
}

//description:   Straight write to module serial port
//input:         str - text to write
//return:        none
void cmd_write(String str) {
  debug_printP( PSTR("Write: "),            LEVEL_DEBUG, 1, 0 );
  for (int i=0; i<str.length(); i++)
    debug_printSafe(str[i],                 LEVEL_DEBUG );
  debug_print( "",                          LEVEL_DEBUG, 0, 1 );
  Serial.print(str);                      //data to send
}
    
//description    Check if CELLv1.0 is on, if not, turn it on
//input:         none
//return:        none
byte cmd_power() {
  debug_printP( PSTR("CMD Power"),          LEVEL_DEBUG, 1, 1 );
    byte tries = 3;
    byte ret = 1;
    while (tries > 0) {
        tries -= 1;
        debug_led();
        debug_inputs();
        
        byte tries_led=3;                 //in case the LED is flashing
        while ((tries_led > 0) and (ret != 0)) {
            ret &= gpio_ledLit();
            tries_led -= 1;
            delay(1000);
        }
        
        debug_printP( PSTR("LED RETURN: "), LEVEL_DEBUG, 1, 0 );
        debug_print( String(ret),           LEVEL_DEBUG, 0, 0 );
        debug_printP( PSTR("(0 1+ LED on, 1 all LEDs off)"), LEVEL_DEBUG, 0, 1 );
        
        if (ret != 0) {
            //ret_status = gpio_statusMR();
            //if (ret_status != 0)
            gpio_turnOn();
        }
        else {
            tries = 0;
        }
    }
    if (ret != 0) {
        debug_printP( PSTR("FATAL ERROR - Module is not turning on (Check power)"), LEVEL_CRITICAL, 1, 1 );
        return 1;
    }
    return 0;
}

/*
int cmd_9600() {
  debug_printP( PSTR("9600"),              LEVEL_DEBUG, 1, 1 );
  byte tries = 3;
  while (tries--) {
mySerial.println("TRIES: " + String(tries));
mySerial.println(" BAUD 115200");
    int ret = cmd_full("at+zbitrate=9600\r", 3, 4000, "");
    util_clearBuffer();
    if (ret == -1) {
      debug_printP( PSTR(" failed attempt"), LEVEL_DEBUG, 1, 1 );
      
      //Check if it's still 115200 baud, this also verifies the modem is communicating
      ret = cmd_full("at+zbitrate?\r", 3, 4000, "");
      debug2(" baud check at 115200: ", ret);
      util_clearBuffer();
      if (ret == -1) {
        Serial.end();
        Serial.begin(115200);
        gpio_toggle();
      }
    }
    else {
      debug_printP( PSTR(" 9600 pass"),    LEVEL_DEBUG, 1, 1 );
      Serial.end();
      Serial.begin(9600);
      delay(1000);
      ret = cmd_full("at+zbitrate?\r", 3, 4000, "");
      debug2(" baud check at 9600: ", ret);
      util_clearBuffer();
      return 0;
    }
  }
mySerial.println("---------------------");
mySerial.println("BAUD CHANGE FAILED!!!");
mySerial.println("---------------------");

  while(1);
  
  return 1;
}
*/

//description    Check if CELLv1.0 is connected to a network, if not, try to connect or fix the issue
//input:         none
//return:        none
byte cmd_connection() {
  debug_printP( PSTR("CMD Connection"),     LEVEL_DEBUG, 1, 1 );
  byte ret = gpio_ledConnected();
  if (ret != 0) {
    ret = info_status();          //check SIM registration status - try to fix or turn on if necessary
    if (ret) {
      debug_printP( PSTR("FATAL ERROR - Module is not connecting to network (Check antenna and SIM card)"), LEVEL_CRITICAL, 1, 1 );
      return 1;
    }
  }
  return 0;
}

//description:   "command" is executed a maximum number of tries with a "delay" in between if fail
//               "buffer" contains valid response matched by "search" or last response if "search not found
//input:         command string, maximum number of tries, delay between each, search string
//return:        0=no issue, 1=error
int cmd_full(String string, byte tries, unsigned long time, String search) {
  if (search == "") {
    search = "OK";
  }
  debug_printP( PSTR("CMD: Full"),          LEVEL_INFO, 1, 1 );
  debug_print ( " ",                        LEVEL_DEBUG, 1, 0);
  for(byte i=0;i<string.length();i++) debug_printSafe(string[i], LEVEL_DEBUG );
  debug_print ( ", " + String(tries) + ", ", LEVEL_DEBUG, 0, 0);
  debug_print ( String(time) + ", ",        LEVEL_DEBUG, 0, 0);
  debug_print ( search,                     LEVEL_DEBUG, 0, 1);
    
  while (tries > 0) {               //"command" is executed a maximum number of tries
    //debug_print( "TRY: " + tries, LEVEL_DEBUG )
    cmd_write(string);            //write to modem

/*
//NOTES:
//======
// 1) Function written in order to not to overrun the hardware serial buffer
//    be careful though, a long response could overwhelm the system
//    watch buffer.length() for long responses
// 2) Could add a buffer check for "search" in order to end
//    the wait period early if response was received
// 3) A quick way to extend the hardware serial buffer is to modify
//    SERIAL_BUFFER_SIZE to 128 in
//    Arduino\hardware\arduino\cores\arduino\HardwareSerial.cpp

    unsigned long timeEnd = millis() + time;
    while (timeEnd > millis()) {
      while (Serial.available()) {
        char tempC = (char)Serial.read(); // add it to the buffer
debug_printSafe(tempC,              LEVEL_DEBUG );
        buffer += tempC;
      }
      delay(50);  //modify this variable depending on the baud rate
    }
*/
    
    delay(time);
    while (Serial.available()) {
      char tempC = (char)Serial.read(); // add it to the buffer
//debug_printSafe(tempC,              LEVEL_DEBUG );
      buffer += tempC;
  
      //debug_printP( PSTR("char: "),           LEVEL_DEBUG, 1, 0 );
      //debug_printSafe(inChar, LEVEL_DEBUG );
      //debug_print( "",                        LEVEL_DEBUG, 0, 1 );
    }
    
    int matchRet = buffer.indexOf(search);
    if (matchRet != -1) {
      debug_printP ( PSTR(" match: "),      LEVEL_DEBUG, 1, 0);
      debug_print( String(matchRet),        LEVEL_DEBUG, 0, 1 );
      return matchRet;
    }
    else {
      debug_printP ( PSTR(" no match"),     LEVEL_DEBUG, 1, 1 );
      tries -= 1;
    }
  }
  return -1;
}
    
/*
//description:   Specialized function to parse HTML response from a web query
//input:         none
//return:        body without header
byte cmd_parseHtmlResponse() {
  debug_printP( PSTR("CMD: Parse HTML Response"), LEVEL_INFO, 1, 1 );


//PORT FROM RASPI, NEED TO COMPOSE FOR ARDUINO
  int matchRet = buffer.indexOf(search);
    if (matchRet != -1) {
      debug_printP ( PSTR(" match: "),      LEVEL_DEBUG, 1, 0);
      debug_print( String(matchRet),        LEVEL_DEBUG, 0, 1 );
      return matchRet;
    }
    else {
      debug_printP ( PSTR(" no match"),     LEVEL_DEBUG, 1, 1 );
      tries -= 1;
    }

  byte matchObj = 0;
  //matchObj = re.search(r'Content-Type:\s+\S+\s+[\n\r]+(.*)[\s\n\r]+\+ZIPSTAT.+', buffer, re.S);
  if (matchObj) {
    //debug_print( " MATCHED (1): " + matchObj.group(1), LEVEL_DEBUG, 1, 1 );
    //return matchObj.group(1);
  }
  else {
    debug_printP( PSTR(" HTML match failed"), LEVEL_INFO, 1, 1 );
    return 0;
  }
}
*/

//-----------------------------------------------------------------------------
// AT COMMANDS ----------------------------------------------------------------
//-----------------------------------------------------------------------------
//NOTE: cmd_full(string, tries, delay, search)

//base AT commands
int at_at() {                                       //Quick status
    return cmd_full("at\r", 3, 4000, "");
}
    
int at_ate1() {                                     //Enable command echo
    return cmd_full("ate1\r", 3, 4000, "");
}
    
int at_ate0() {                                     //Disable command echo
    return cmd_full("ate0\r", 3, 4000, "");
}
    
int at_creg() {                                     //Network registration status
    return cmd_full("at+creg?\r", 3, 4000, "CREG");
}
    
int at_cops() {                                     //List available networks
    return cmd_full("at+cops=?\r", 3, 300000, "COPS");
}
    
int at_cnti() {                                     //Current tech
    return cmd_full("at*cnti?\r", 3, 4000, "");
}
    
int at_csq() {                                      //Signal Quality
    return cmd_full("at+csq\r", 3, 4000, "");
}

//SMSes
int at_cmgf() {                                     //set SMS to plaintext mode
    return cmd_full("at+cmgf=1\r", 3, 4000, "");
}
    
int at_cmgs(String num) {                           //set destination phone number, format "+12223334444"
    return cmd_full("at+cmgs=\"" + num + "\"\r", 3, 4000, ">");
}

int at_cnmiOut() {                                  //set SMS to pass incoming directly to output
    return cmd_full("at+cnmi=1,2,0,0,0\r", 3, 4000, "");
}
    
int at_cmgsMem() {                                  //set SMS to pass incoming directly to memory
    return cmd_full("at+cnmi=1,1,0,0,0\r", 3, 4000, "");
}

int at_cmgr(byte ind) {                             //Read message at index 0
    return cmd_full("at+cmgr=" + String(ind) + "\r", 3, 4000, "");
}
    
int at_cmgd(byte ind) {                             //Delete message at index 0
    return cmd_full("at+cmgd=" + String(ind) + "\r", 3, 4000, "");
}
    
int at_cmgl() {                                     //Read all message
    return cmd_full("at+cmgl=4\r", 3, 4000, "");
}

//TCP/UDP
int at_zipcallOpen() {                              //start call - state 1 open
    return cmd_full("at+zipcall=1\r", 3, 10000, "ZIPCALL:");
}

int at_zipcallClose() {                             //stop call - state 0 close
    return cmd_full("at+zipcall=0\r", 3, 4000, "");
}

int at_zipopen(byte socket, byte type, String ip, int port) {            //socket id,type(UDP=1,TCP=0),hostname/IP,Port
    return cmd_full("at+zipopen=" + String(socket) + "," + String(type) + "," + ip + "," + String(port) + "\r", 3, 4000, "ZIPSTAT:");
}
    
int at_zipsend(byte socket) {                       //send data
    return cmd_full("at+zipsend=" + String(socket) + "\r", 3, 4000, ">");
}
    
int at_zipwrite(String msg) {
    cmd_write(msg);
    //util_clearBuffer();
    return cmd_full("\x1A", 1, 10000, "");
}


//-----------------------------------------------------------------------------
// ARDUINO SPECIFIC -----------------------------------------------------------
//-----------------------------------------------------------------------------

//description:   Clear the buffer
//input:         none
//return:        none
void util_clearBuffer() {
    debug_printP( PSTR("UTIL: Clear Buffer"), LEVEL_DEBUG, 1, 1 );
    buffer = "";                              // clear the buffer
}

//description:   Print buffer
//input:         none
//return:        none
void util_printBuffer() {
  debug_printP( PSTR("UTIL: Print Buffer: "), LEVEL_DEBUG, 1, 0 );
  for (int i=0; i<buffer.length(); i++)
    debug_printSafe(buffer[i],              LEVEL_DEBUG );
  debug_print( "",                          LEVEL_DEBUG, 0, 1 );
      
  //debug_printP( PSTR("UTIL: Print Buffer"), LEVEL_DEBUG, 1, 1 );
  //debug_print( buffer,                    LEVEL_DEBUG, 1, 1 );
  //debug_printP( PSTR("-----"),            LEVEL_DEBUG, 1, 1 );
}

void util_time() {
  //DEBUG TIME SINCE START (SECS)
  time = millis()/1000;            // time since program started
  debug_printP( PSTR("Time: "),            LEVEL_DEBUG, 1, 0 );
  debug_print( String(time),               LEVEL_DEBUG, 0, 0 );
  debug_printP( PSTR(" (secs)"),           LEVEL_DEBUG, 0, 1 );
}


//-----------------------------------------------------------------------------
// COMBINED AT COMMANDS -------------------------------------------------------
//-----------------------------------------------------------------------------

//description:   Full SMS send
//input:         num ("+12223334444") - phone number, msg - message to send
//return:        none
byte at_sms(String num, String msg) {
    ret = at_cmgf();
    debug_printP( PSTR("CMGF - Plaintext"), LEVEL_DEBUG, 1, 1 );
    debug_printP( PSTR("---> CMD RET --->: "), LEVEL_DEBUG, 1, 0);
    debug_print( String(ret),              LEVEL_DEBUG, 0, 1);
    util_printBuffer();
    if (ret == -1) {
        debug_printP( PSTR("SMS Failed: cmgf"), LEVEL_ERROR, 1, 1 );
        return 1;
    }
    util_clearBuffer();
        
    ret = at_cmgs(num);
    debug_printP( PSTR("CMGS - Phone #"),  LEVEL_DEBUG, 1, 1 );
    debug_printP( PSTR("---> CMD RET --->: "), LEVEL_DEBUG, 1, 0);
    debug_print( String(ret),              LEVEL_DEBUG, 0, 1);
    util_printBuffer();
    if (ret == -1) {
        debug_printP( PSTR("SMS Failed: cmgs"), LEVEL_ERROR, 1, 1 );
        return 1;
    }
    util_clearBuffer();
        
    cmd_write(msg);
    debug_printP( PSTR("WRITE"),           LEVEL_DEBUG, 1, 1 );
    ret = cmd_full("\x1A", 1, 10000, "");
    if (ret == -1) {
        debug_printP( PSTR("SMS Failed: finish"), LEVEL_ERROR, 1, 1 );
        return 1;
    }
    util_printBuffer();
    util_clearBuffer();
    
    return 0;
}
